#  Copyright (C) 2007 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

if $0 == __FILE__
	puts "Nothing to see here..."
	puts "Did you meant to run plugins_tests.rb?"
	exit 1
end

require File.expand_path( File.dirname( __FILE__ ) + "/../utils/htmlentities" )
require "test/unit"

class PluginTests < Test::Unit::TestCase

	def self.build_suite( plugin )
		@@pluginPluginTests = plugin
		eval(
			"class #{plugin.class.name}Tests < PluginTests\n" + \
				"@@plugin = @@pluginPluginTests\n" + \
				"def self.plugin()\n" + \
					"return @@plugin\n" + \
				"end\n" + \
			"end"
		)
		return eval( "#{plugin.class.name}Tests.suite()" )
	end

	def plugin()
		self.class.plugin()
	end

	def check_suggestions( suggestions, title, artist, site_name )
		assert_not_nil( suggestions, "No suggestions for #{title} by #{artist} returned by #{site_name}" )
		assert( ! suggestions.empty?(), "Empty suggestions list for #{title} by #{artist} returned by #{site_name}" )
		suggestions.each() do |sugg|
			["artist", "title"].each() do |key|
				assert_nil(
					sugg[key].downcase().index( "lyrics" ),
					"Suggestions for #{title} by #{artist} returned by #{site_name} contains the 'lyrics' word (#{sugg[key]})"
				)
				assert(
					sugg[key] == HTMLEntities.decode( sugg[key] ),
					"Suggestions for #{title} by #{artist} returned by #{site_name} contains unexpected HTML entities (#{sugg[key]})"
				)
			end
		end
	end

	def check_lyrics_data( ld, title, artist, site_name )
		["artist", "title", "album"].each() do |key|
			assert_nil(
				ld[key].downcase().index( "lyrics" ),
				"Lyrics metadata for #{title} by #{artist} returned by #{site_name} contains the 'lyrics' word"
			)
		end
		lyrics = ld["lyrics"]
		assert_not_nil( lyrics, "No lyrics for #{title} by #{artist} returned by #{site_name}" )
		assert( ! lyrics.strip().empty?, "Empty lyrics for #{title} by #{artist} returned by #{site_name}" )
		md = /<\/?(a|h[1-6]|hr|p|br|body|html|span|div|br|script|!?--) *\/?>*/i.match( lyrics )
		assert_nil( md, "Lyrics for #{title} by #{artist} returned by #{site_name} contains unexpected HTML tags: #{md ? md[1] : ""}" )
	end

	def test_lyrics_direct_search()
		puts "\nTesting #{plugin().site_name()} lyrics direct search..."
		self.class.get_test_data( plugin() ).each() do |song_data|
			artist = song_data["artist"]
			album = song_data["album"]
			title = song_data["title"]
			if ( plugin().build_lyrics_fetch_data( artist, title, album, nil )["url"] )
				puts " - searching lyrics for #{title} by #{artist}"
				ld = plugin().lyrics_direct_search( artist, title, album, nil )
				check_lyrics_data( ld, title, artist, plugin.site_name() )
			else
				puts " - skipping lyrics search for #{title} by #{artist} (no direct fetch data received)"
				break
			end
		end
	end

	def test_lyrics_from_suggestions()
		puts "\nTesting #{plugin().site_name()} lyrics from suggestions..."
		self.class.get_test_data( plugin() ).each() do |song_data|
			artist = song_data["artist"]
			album = song_data["album"]
			title = song_data["title"]
			stitle = song_data["stitle"]
			puts " - searching suggestions for #{stitle} by #{artist}"
			suggestions = plugin().suggestions( artist, stitle, album, nil )
			check_suggestions( suggestions, stitle, artist, plugin.site_name() )
			puts " - searching lyrics to #{title} by #{artist} in suggestions"
			ld = plugin().lyrics_from_suggestions( artist, title, album, nil, suggestions )
			check_lyrics_data( ld, title, artist, plugin.site_name() )
		end
	end

	def test_lyrics_full_search()
		puts "\nTesting #{plugin().site_name()} lyrics full search..."
		self.class.get_test_data( plugin() ).each() do |song_data|
			artist = song_data["artist"]
			album = song_data["album"]
			title = song_data["title"]
			puts " - searching lyrics for #{title} by #{artist}"
			ld = plugin().lyrics_full_search( artist, title, album, nil )
			check_lyrics_data( ld, title, artist, plugin.site_name() )
		end
	end

	def test_no_lyrics_found()
		puts "\nTesting #{plugin().site_name()} forced retrieval fail..."
		artist = "Fake Artist"
		album = "Fake Album"
		title = "Fake Title"
		puts " - searching lyrics for #{title} by #{artist}"
		ld = plugin().lyrics_full_search( artist, title, album, nil )
		assert_nil( ld["lyrics"], "Unexpected lyrics returned for #{title} by #{artist} by #{plugin.site_name()}" )
	end

	def self.get_test_data( plugin )
		songs_data = plugin.get_test_data()
		songs_data.each() { |song_data| song_data["stitle"] = song_data["title"] if ( ! song_data.include?( "stitle" ) ) }
		return songs_data
	end

end
